@@ -13,6 +13,8 @@ var _ = require('underscore'),
     Step = require('step'),
     utils = require('./util.js');
 
+var path_sep = process.platform === 'win32' ? '\\' : '/';
+
 // Known SRS values
 var SRS = {
     'WGS84': '+proj=longlat +ellps=WGS84 +datum=WGS84 +no_defs',
@@ -32,8 +34,7 @@ var pool = require('generic-pool').Pool({
     max: 5
 });
 
-function download(l, filepath, callback) {
-    var url = l.Datasource.file;
+function download(url, filepath, callback) {
     var dl = filepath + '.download';
     // If this file is already being downloaded, attach the callback
     // to the existing EventEmitter
@@ -66,6 +67,26 @@ function download(l, filepath, callback) {
     }
 }
 
+// Retrieve a remote copy of a resource only if we don't already have it.
+function localize(url, filepath, next) {
+    path.exists(filepath, function(exists) {
+        if (exists) {
+            next(null, filepath);
+        } else {
+            utils.mkdirP(path.dirname(filepath), 0755, function(err) {
+                if (err && err.code !== 'EEXIST') {
+                    next(err);
+                } else {
+                    download(url, filepath, function(err, filepath) {
+                        if (err) return next(err);
+                        next(null, filepath);
+                    });
+                }
+            });
+        }
+    });
+}
+
 // Generate the cache path for a given URL.
 function cachepath(location) {
     var uri = url.parse(location);
@@ -195,6 +216,25 @@ function fixSRS(obj) {
         .value()) obj.srs = SRS['900913'];
 }
 
+function checkTTL(cache, l) {
+    var file = l.Datasource.file;
+    var ttl = l.Datasource.ttl
+
+    if (!url.parse(file).protocol) return;
+
+    var filepath = path.join(cache, cachepath(file));
+    fs.stat(filepath, function(err, stats) {
+        if (err && err.code != 'ENOENT') return console.warn(err);
+
+        var msttl = parseInt(ttl) * 1000;
+        if (err || Date.now() > (new Date(stats.mtime).getTime() + msttl)) {
+            download(file, filepath, function(err, filepath){
+                if (err) return console.warn(err);
+            });
+        }
+    });
+}
+
 // Resolve and process all externals for an MML file.
 function resolve(options, callback) {
     if (typeof callback !== 'function') throw new Error('Second argument must be a callback');
@@ -228,7 +268,7 @@ function resolve(options, callback) {
             var uri = url.parse(s);
 
             // URL, download.
-            if (uri.protocol) {
+            if (uri.protocol && (uri.protocol == 'http:' || uri.protocol == 'https:')) {
                 return (new get(s)).asBuffer(function(err, data) {
                     if (err) return next(err);
 
@@ -241,7 +281,7 @@ function resolve(options, callback) {
             }
 
             // File, read from disk.
-            if (uri.pathname[0] !== '/') {
+            if (uri.pathname[0] !== path_sep) {
                 uri.pathname = path.join(base, uri.pathname);
             }
             fs.readFile(uri.pathname, 'utf8', function(err, data) {
@@ -258,6 +298,9 @@ function resolve(options, callback) {
         resolved.Layer.forEach(function(l, index) {
             if (!l.Datasource || !l.Datasource.file) return next();
 
+            // TODO find  better home for this check.
+            if (l.Datasource.ttl) checkTTL(cache, l);
+
             var name = l.name || 'layer-' + index,
                 uri = url.parse(encodeURI(l.Datasource.file)),
                 pathname = decodeURI(uri.pathname),
@@ -267,8 +310,8 @@ function resolve(options, callback) {
             // l, extname, and more is all up-one-level.
             //
             // `file`: filename to be symlinked in place to l.Datasource.file
-            var symlink = function(file) {
-                if (!file) return next();
+            var symlink = function(file, cb) {
+                if (!file) return cb();
 
                 switch (extname.toLowerCase()) {
                 // Unzip and symlink to directory.
@@ -279,12 +322,12 @@ function resolve(options, callback) {
                             name,
                             path.basename(file, path.extname(file)) + '.shp');
                     path.exists(l.Datasource.file, function(exists) {
-                        if (exists) return next();
+                        if (exists) return cb();
                         unzip(file, function(err, file) {
-                            if (err) return next(err);
+                            if (err) return cb(err);
                             utils.forcelink(path.dirname(file),
                                 path.dirname(l.Datasource.file),
-                                next);
+                                cb);
                         });
                     });
                     break;
@@ -294,42 +337,29 @@ function resolve(options, callback) {
                         path.join(base, 'layers', name, path.basename(file));
                     utils.forcelink(
                         path.dirname(file),
-                        path.dirname(l.Datasource.file), next);
+                        path.dirname(l.Datasource.file), cb);
                     break;
                 // Symlink files
                 default:
                     l.Datasource.file =
                         path.join(base, 'layers', name + extname);
-                    utils.forcelink(
-                        file,
+                    utils.forcelink( file,
                         l.Datasource.file,
-                        next);
+                        cb);
                     break;
                 }
             };
 
             // URL.
-            if (uri.protocol) {
+            if (uri.protocol && (uri.protocol == 'http:' || uri.protocol == 'https:')) {
                 var filepath = path.join(cache, cachepath(l.Datasource.file));
-                path.exists(filepath, function(exists) {
-                    if (exists) {
-                        symlink(filepath);
-                    } else {
-                        utils.mkdirP(path.dirname(filepath), 0755, function(err) {
-                            if (err && err.code !== 'EEXIST') {
-                                next(err);
-                            } else {
-                                download(l, filepath, function(err, filepath) {
-                                    if (err) return next(err);
-                                    symlink(filepath);
-                                });
-                            }
-                        });
-                    }
+                localize(uri.href, filepath, function(err, file) {
+                    if (err) return next(err);
+                    symlink(file, next)
                 });
             // Absolute path.
-            } else if (pathname && pathname[0] === '/') {
-                symlink(pathname);
+            } else if (pathname && pathname[0] === path_sep) {
+                symlink(pathname, next);
             // Local path.
             } else {
                 l.Datasource.file = path.resolve(path.join(base, pathname));
@@ -349,18 +379,41 @@ function resolve(options, callback) {
                 var dbs = d.attachdb.split(',');
                 Step(function() {
                     var group = this.group();
-                    for (i = 0; i < dbs.length; i++) (function(next) {
+                    for (var i = 0; i < dbs.length; i++) (function(next) {
                         if (!dbs[i]) {
                             return next();
                         }
 
-                        var file = dbs[i].split('@').pop();
-                        if (file[0] != '/') {
-                            file = path.resolve(path.join(base, file));
-                            dbs[i] = dbs[i].split('@').shift() + '@' + file;
+                        var file = url.parse(dbs[i].split('@').pop());
+                        var pathname = file.pathname;
+                        var extname = path.extname(pathname);
+                        var alias = dbs[i].split('@').shift();
+                        var name = (l.name || 'layer-' + index) + '-attach-' + alias;
+                        var index = i;
+
+                        var symlink = function(filepath, cb) {
+                            var filename = path.join(base, 'layers', name + extname);
+                            dbs[index] =  alias + '@' + filename;
+                            utils.forcelink(filepath, filename, cb);
+                        };
+
+                        // URL.
+                        if (file.protocol) {
+                            var filepath = path.join(cache, cachepath(file.href));
+                            localize(file.href, filepath, function(err) {
+                                if (err) return next(err);
+                                symlink(filepath, next);
+                            });
+                        }
+                        // Absolute path.
+                        else if (pathname[0] === path_sep) {
+                            symlink(pathname, next);
+                        }
+                        // Local path.
+                        else {
+                            dbs[index] =  alias + '@' + path.join(base, pathname);
+                            next();
                         }
-                        next();
-                        
                     })(group());
                 }, function(err) {
                     if (err) throw err;
